rem *********************************************
rem global variables
rem *********************************************
dim title			as string
dim viewname		as string
dim view			as integer
dim typecfg			as object
dim filtercfg		as object

rem *********************************************
rem generates a html view of all types, which
rem are available by this office installation.
rem *********************************************
sub main

	rem ask user for file name
	viewname = inputbox("Please specify the file name of the html view, which should be generated by this script.", "Needed", "c:\temp\filter_view.html")
	if viewname = "" then
		msgbox "Please specify a valid file name."
		exit sub
	endif

	rem should it be overwritten?
	if fileExists(viewname) then
		rem 4 => yes/no
		dim decision as integer
		decision = msgbox("This file already exists. Should it be overwritten?", 4, "Conflict")
		if decision <> 6 then
			exit sub
		endif
		rem remove old file!
		kill viewname
	endif

	rem ask user for a title
	title = inputbox("Please type in the name of used office version.", "Needed")
	if title = "" then
		msgbox "Please specify a valid office version."
		exit sub
	endif

	rem open the output file
	view = FreeFile
	open viewname for binary as #view

	rem get all needed resources
	typecfg		= createUnoService("com.sun.star.document.TypeDetection")
	filtercfg	= createUnoService("com.sun.star.document.FilterFactory")

	if isNull(typecfg) or isNull(filtercfg) then
		msgbox "Could not locate types/filter configuration data."
		exit sub
	endif

	rem fill it
rem    generateTypeView(#view, title)
    generateFilterView(#view, title)

	rem close the output file
	close #view

end sub

rem *********************************************
rem uses configuration data to fill the html view
rem *********************************************
sub generateFilterView(	#view	as integer	, _
						title	as string	)

    dim html as string
    html = ""

	rem generate html header
	html = html + "<html><head><title>"+title+"</title></head>"

	rem generate body
    html = html + "<body><h1>Filters for " + title + "</h1>"

rem    dim filters()   as string
rem     filters() = queryFiltersForModule("com.sun.star.text.TextDocument")
rem     for f=0 to ubound(filters()) step 1
rem         html = html + "'" + filters(f) + "'<br>"
rem     next f

    html = html + "<ul>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.text.TextDocument"                  + chr(34) + ">com.sun.star.text.TextDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.text.WebDocument"                   + chr(34) + ">com.sun.star.text.WebDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.text.GlobalDocument"                + chr(34) + ">com.sun.star.text.GlobalDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.sheet.SpreadsheetDocument"          + chr(34) + ">com.sun.star.sheet.SpreadsheetDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.drawing.DrawingDocument"            + chr(34) + ">com.sun.star.drawing.DrawingDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.presentation.PresentationDocument"  + chr(34) + ">com.sun.star.presentation.PresentationDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.chart.ChartDocument"                + chr(34) + ">com.sun.star.chart.ChartDocument</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.formula.FormulaProperties"          + chr(34) + ">com.sun.star.formula.FormulaProperties</a></li>"
    html = html + "<li><a href=" + chr(34) + "#com.sun.star.sdb.OfficeDatabaseDocument"         + chr(34) + ">com.sun.star.sdb.OfficeDatabaseDocument</a></li>"
    html = html + "</ul>"

    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.text.TextDocument"                )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.text.WebDocument"                 )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.text.GlobalDocument"              )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.sheet.SpreadsheetDocument"        )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.drawing.DrawingDocument"          )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.presentation.PresentationDocument")
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.chart.ChartDocument"              )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.formula.FormulaProperties"        )
    html = html + getTableForModule(typecfg, filtercfg, "com.sun.star.sdb.OfficeDatabaseDocument"       )

	html = html + "</body></head>"

	put #view,1,html

end sub

sub generateTypeView(  #view   as integer  , _
                       title   as string   )


    rem generate html header
rem    html = html + "<html><head><title>"+title+"</title></head>"
    rem generate body
rem    html = html + "<body><h1>Types for " + title + "</h1><ul>"
	html = ""
    html = html + getTypeView()
rem    html = html + "</body></head>"

    put #view,1,html

end sub

function locateType(names() as string, sname as string) as integer
	dim found as integer
	dim s1    as integer
	found = -1
	for s1=0 to ubound(names()) step 1
		if names(s1) = sname then
			found = s1
		rem	exit for
		endif
	next s1
	locateType = found
end function

function getTypeView() as string

	dim result   	 as string
	dim filters()  	 as string
	dim fprops() 	 as variant
	dim f        	 as integer
	dim fp       	 as integer
	dim tprops() 	 as variant
	dim t        	 as integer
	dim tp       	 as integer
	dim fname        as string
	dim tname        as string
	dim uiname       as string
	dim extensions() as string
	dim flags        as long
	dim i        	 as integer
	dim e        	 as integer
	dim typecount  	 as integer
	dim import       as boolean
	dim export       as boolean
	dim internal     as boolean

	dim t_intnames  (500) as string	
	dim t_uinames   (500) as string	
	dim t_extensions(500) as string
	dim t_flags     (500) as long
	
	typecount = 0
	filters() = filtercfg.getElementNames()
	for f=0 to ubound(filters()) step 1
		
		fname    = filters(f)
		fprops() = filtercfg.getByName(fname)
		
		for fp=0 to ubound(fprops()) step 1
			if fprops(fp).Name = "Type" then
				tname = fprops(fp).Value
			endif
			if fprops(fp).Name = "Flags" then
				flags = fprops(fp).Value
			endif
		next fp		

		tprops() = typecfg.getByName(tname)

		for tp=0 to ubound(tprops()) step 1
			if tprops(tp).Name = "UIName" then
				uiname = tprops(tp).Value
			endif
			if tprops(tp).Name = "Extensions" then
				extensions() = tprops(tp).Value
			endif
		next tp		

		if tname <> "" then
			i = locateType(t_intnames(), tname)
			if i<>-1 then
				t_intnames  (i) = tname
				t_uinames   (i) = uiname
				t_flags     (i) = flags
				for e=0 to ubound(extensions()) step 1
					t_extensions(i) = t_extensions(i)+";"+extensions(e)
				next e
			else
				i         = typecount
				typecount = typecount+1
				t_intnames  (i) = tname
				t_uinames   (i) = uiname
				t_flags     (i) = flags
				for e=0 to ubound(extensions()) step 1
					t_extensions(i) = t_extensions(i)+";"+extensions(e)
				next e
			endif
		endif
		
	next f

	result = ""
	for t=0 to typecount step 1
		result  = result+"'"+t_uinames(t)+"'"+chr(9)
		result  = result+t_extensions(t)+chr(9)
		import  = false
		export  = false
		internal= false
		if t_flags(t) and 1 then
			import = true
		endif
		if t_flags(t) and 2 then
			export = true
		endif
		if t_flags(t) and 8 then
			internal = true
		endif
		if t_flags(t) and 4096 then
			internal = true
		endif
		if t_flags(t) and 8192 then
			internal = true
		endif
		if import then
			result = result+"IMPORT"+chr(9)
		endif
		if export then
			if not import then
				result = result+chr(9)
			endif
			result = result+"EXPORT"+chr(9)
		endif
		if internal then
			if not import then
				result = result+chr(9)
			endif
			if not export then
				result = result+chr(9)
			endif
			result = result+"INTERNAL"
		endif
		result = result+chr(13)
	next t
		
	getTypeView = result

end function

function queryFiltersForModule( module as string ) as variant

	dim f			as integer
	dim fp			as integer
	dim r			as integer
	dim filters()	as string
	dim fprops()	as variant
	dim result()	as string
	dim	docservice	as string

	filters()	= filtercfg.getElementNames()
	result()	= filters()

	for f=0 to ubound(filters()) step 1
		fprops() = filtercfg.getByName(filters(f))
		for fp=0 to ubound(fprops()) step 1
			if fprops(fp).Name = "DocumentService" then
				docservice = fprops(fp).Value
				if docservice = module then
					result(r) = filters(f)
					r = r + 1
				endif
			endif
		next fp
	next f

	if r>0 then
		redim preserve result(r-1) as string
	else
		redim result() as string
	endif
	queryFiltersForModule = result()

end function

rem *********************************************
rem return a html formated table, which contains
rem only filters of the required module
rem *********************************************
function getTableForModule(typecfg         as object ,_
                           filtercfg       as object ,_
                           required_module as string ) as string

	rem get names of all well known types/filters
	dim types()   as variant
	dim filters() as variant
	types()   = typecfg.getElementNames()
	filters() = queryFiltersForModule(required_module)

	rem initialize progress
	dim frame 	 as object
	dim progress as object
	frame = StarDesktop.getCurrentFrame()
	if not isnull(frame) then
		progress = frame.createStatusIndicator()
	endif

	if not isnull(progress) then
		progress.start("scan filters for module "+required_module+" ...", ubound(filters()))
	endif

	dim table 	 as string
	dim uname  	 as string
	dim aname  	 as string
	dim module   as string
	dim import   as string
	dim export   as string
	dim intern   as string
	dim flags 	 as long
	dim f 		 as integer
	dim props()  as variant
	dim count	 as integer
    dim sflags   as string

	rem step over all(!) filters
	rem but use only filters of the
	rem required module ...
	count = 0
	for f=0 to ubound(filters()) step 1

		if not isnull(progress) then
			progress.setValue(f+1)
		endif

		import  = "-"
		export  = "-"
		options = "-"
		visible = "-"
		flagval = ""
		aname   = filters(f)

		fprops() = filtercfg.getByName(aname)
		for p=0 to ubound(fprops()) step 1
			if fprops(p).Name = "UIName" then
				uname = fprops(p).Value
			elseif fprops(p).Name = "Flags" then
				flags   = fprops(p).Value
                flagval = flags_int2str(flags)
                
				if flags and 1 then
					import = "X"
				endif
				if flags and 2 then
					export = "X"
				endif
				if flags and 128 then
					options = "X"
				endif
                if ((flags and 4096) or (flags and 8192)) then
                    visible = "X"
                endif
                
			elseif fprops(p).Name = "DocumentService" then
				module = fprops(p).Value
				if module <> required_module then
					exit for
				endif
			endif
		next p

		if module = required_module then
			count = count +1
	 		table = table + "<tr>"
	 		table = table + "<td>" + count + "</td>"
	 		table = table + "<td>" + chr(34) + uname  + chr(34) + "</td>"
	 		table = table + "<td>" + chr(34) + aname  + chr(34) + "</td>"
	 		table = table + "<td>" + import  + "</td>"
	 		table = table + "<td>" + export  + "</td>"
	 		table = table + "<td>" + options + "</td>"
	 		table = table + "<td>" + visible + "</td>"
	 		table = table + "<td>" + flagval + "</td>"
	 		table = table + "</tr>"
		endif

	next f

	if not isnull(progress) then
		progress.end()
	endif

	rem generate header and footer
    dim html_ret as string
    html_ret = ""
    html_ret = html_ret + "<p><a name=" + chr(34) + required_module + chr(34) + "><h1>" + required_module + "(" + count + ") </h1></a>"
    html_ret = html_ret + "<table border=1><tr>"+ _
    					  "<td><b>Nr.</b></td>"+ _
    					  "<td><b>Localized Name</b></td>"+ _
    					  "<td><b>API Name</b></td>"+ _
    					  "<td><b>IMPORT</b></td>"+ _
						  "<td><b>EXPORT</b></td>"+ _
						  "<td><b>SHOWS DIALOG</b></td>"+ _
						  "<td><b>HIDDEN FROM UI</b></td>"+ _
						  "<td><b>ALL FLAGS</b></td>"+ _
						  "</tr>"
    html_ret = html_ret + table + "</table>"

    getTableForModule = html_ret

end function

function flags_int2str(flags as long) as string

    dim cflags as long
    dim strval as string

    cflags = flags
	strval = ""

    if cflags and 1 then
        strval = strval + " IMPORT"
        cflags = cflags-1
    endif
    if cflags and 2 then
        strval = strval + " EXPORT"
        cflags = cflags-2
    endif
    if cflags and 4 then
        strval = strval + " TEMPLATE"
        cflags = cflags-4
    endif
    if cflags and 8 then
        strval = strval + " INTERNAL"
        cflags = cflags-8
    endif
    if cflags and 16 then
        strval = strval + " TEMPLATEPATH"
        cflags = cflags-16
    endif
    if cflags and 32 then
        strval = strval + " OWN"
        cflags = cflags-32
    endif
    if cflags and 64 then
        strval = strval + " ALIEN"
        cflags = cflags-64
    endif
    if cflags and 128 then
        strval = strval + " USESOPTIONS"
        cflags = cflags-128
    endif
    if cflags and 256 then
        strval = strval + " DEFAULT"
        cflags = cflags-256
    endif
    if cflags and 512 then
        strval = strval + " EXECUTABLE"
        cflags = cflags-512
    endif
    if cflags and 1024 then
        strval = strval + " SUPPORTSSELECTION"
        cflags = cflags-1024
    endif
    if cflags and 2048 then
        strval = strval + " MAPTOAPPPLUG"
        cflags = cflags-2048
    endif
    if cflags and 4096 then
        strval = strval + " NOTINFILEDIALOG"
        cflags = cflags-4096
    endif
    if cflags and 8192 then
        strval = strval + " NOTINCHOOSER"
        cflags = cflags-8192
    endif
    if cflags and 16384 then
        strval = strval + " ASYNCHRON"
        cflags = cflags-16384
    endif
    if cflags and 32768 then
        strval = strval + " CREATOR"
        cflags = cflags-32768
    endif
    if cflags and 65536 then
        strval = strval + " READONLY"
        cflags = cflags-65536
    endif
    if cflags and 131072 then
        strval = strval + " NOTINSTALLED"
        cflags = cflags-131072
    endif
    if cflags and 262144 then
        strval = strval + " CONSULTSERVICE"
        cflags = cflags-262144
    endif
    if cflags and 524288 then
        strval = strval + " 3RDPARTYFILTER"
        cflags = cflags-524288
    endif
    if cflags and 1048576 then
        strval = strval + " PACKED"
        cflags = cflags-1048576
    endif
    if cflags and 2097152 then
        strval = strval + " SILENTEXPORT"
        cflags = cflags-2097152
    endif
    if cflags and 4194304 then
        strval = strval + " BROWSERPREFERED"
        cflags = cflags-4194304
    endif
    if cflags and 8388608 then
        strval = strval + " !!! ATTENTION: 8388608  currently unsupported !!!"
        cflags = cflags-8388608
    endif
    if cflags and 268435456 then
        strval = strval + " PREFERED"
        cflags = cflags-268435456
    endif

    if cflags <> 0 then
        strval = strval + " !!! ATTENTION: unsupported flag ["+cflags+"] detected !!!"
    endif

    flags_int2str = strval

end function
